@@ -1,4 +1,5 @@
 import time
+import json
 
 from boto.s3.connection import S3Connection, OrdinaryCallingFormat
 
@@ -56,35 +57,34 @@ def get_headers(metadata, headers={}):
     :param headers: (optional) S3 compatible HTTP headers.
     """
 
-    def encode_s3_header(header_key, header_value):
-        # because rfc822 http headers disallow _ in names, IA-S3 will
-        # translate two hyphens in a row (--) into an underscore (_).
-        header_key = header_key.replace('_', '--')
-        try:
-            encoded_header_key = header_key.encode('utf-8')
-            encoded_header_value = header_value.encode('utf-8')
-        except AttributeError:
-            encoded_header_key = header_key
-            encoded_header_value = header_value
-        return (encoded_header_key, encoded_header_value)
-
     for meta_key, meta_value in metadata.iteritems():
-        try:
-            # Convert metadata items into multiple header fields if
-            # the item's value is iterable, and append to the
-            # headers dict.
-            if isinstance(meta_value, basestring):
-                meta_value=[meta_value]
-            for i, value in enumerate(meta_value):
-                s3_header_key = 'x-archive-meta{0:02d}-{1}'.format(i, meta_key)
-                encoded_header = encode_s3_header(s3_header_key, value)
-                if encoded_header:
-                    header_key, header_value = encoded_header
-                    headers[header_key] = header_value
-        except TypeError:
-            s3_header_key = 'x-archive-meta-{0}'.format(meta_key)
-            encoded_header = encode_s3_header(s3_header_key, value)
-            if encoded_header:
-                header_key, header_value = encoded_header
-                headers[header_key] = header_value
-    return headers
+
+        # Encode arrays into JSON strings because Archive.org does not 
+        # yet support complex metadata structures in 
+        # <identifier>_meta.xml.
+        if isinstance(meta_value, dict):
+            meta_value = json.dumps(meta_value)
+
+        # Convert the metadata value into a list if it is not already
+        # iterable.
+        if not hasattr(meta_value, '__iter__'):
+                meta_value = [meta_value]
+
+        # Convert metadata items into HTTP headers and add to 
+        # ``headers`` dict.
+        for i, value in enumerate(meta_value):
+            if not value:
+                continue
+            s3_header_key = 'x-archive-meta{0:02d}-{1}'.format(i, meta_key)
+
+            # because rfc822 http headers disallow _ in names, IA-S3 will
+            # translate two hyphens in a row (--) into an underscore (_).
+            s3_header_key = s3_header_key.replace('_', '--')
+            try:
+                if not headers.get(s3_header_key):
+                    headers[s3_header_key] = value.encode('utf-8')
+            except AttributeError:
+                if not headers.get(s3_header_key):
+                    headers[s3_header_key] = value
+
+    return dict((k,v) for k,v in headers.items())
